//Demo application to Read and Write RTC & EEPROM using Toradex new library for VFxx and iMX6xx
#include <windows.h>
#include "I2c.h"

typedef enum
{
	RTC_SELECTION ='a', 
	EEPROM_SELECTION = 'b',
	EXIT_SELECTION = 'c',
}SELECTION;

//******************************************************************************
/// RTC demo for testing i2c \n
/// @param[in]  i2c library handle for Selected I2C channel ("I2C1" to "I2C4")
void rtcDemoOptionSelected(HANDLE i2c)
{
	const char *rtcDay[8] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
	BYTE rtcData[8] = {0};														///< I2c Data receive buffer
	BYTE sixBitMask = 0x3F;														///< Mask bit 
	BYTE fiveBitMask = 0x1F;													///< Mask bit
	BYTE threeBitMask = 0x07;													///< Mask bit
	BYTE sevenBitMask = 0x7F;													///< Mask bit
	BYTE rtcAddress = 0x68;														///< Address of RTC
	DWORD returnValue = 0;
	SYSTEMTIME dateTime;
	if(i2c == NULL)																///< Check for valid i2c handle
	{
		printf("I2CDemo Error: Invalid i2c handle\n");
	}
	if(!I2c_SetConfigInt(i2c, L"SlaveAddr", rtcAddress, StoreVolatile))			///< Set Slave address for RTC
	{
		printf("I2CDemo Error: I2c_SetConfigInt Slave address\n");
	}
	returnValue = I2c_Read(i2c, (DWORD*) rtcData, 8);							///< Read i2c channel
	if(returnValue == I2C_RW_FAILURE)											///< Check for number of bytes read
	{
		printf("I2CDemo Error: I2c_Read\n");
	}
	else
	{
		dateTime.wYear = (WORD)rtcData[6];
		dateTime.wMonth = (WORD)(rtcData[5] & fiveBitMask);
		dateTime.wDay = (WORD)(rtcData[4] & sixBitMask);
		dateTime.wHour = (WORD)(rtcData[2] & sixBitMask);
		dateTime.wMinute = (WORD)(rtcData[1] & sevenBitMask);
		dateTime.wSecond = (WORD)(rtcData[0] & sevenBitMask);
		dateTime.wDayOfWeek = (WORD)((rtcData[3] & threeBitMask) - 1);

		printf("Day is %s\n", rtcDay[dateTime.wDayOfWeek]);   
		printf("Today's date is %x/0%x/%x (DD/MM/YY)\n", dateTime.wDay, dateTime.wMonth, dateTime.wYear);
		printf("Time is %x:%x:%x (HH:MM:SS) \n", dateTime.wHour, dateTime.wMinute, dateTime.wSecond);
	}
}

//******************************************************************************
/// External EEPROM demo for testing i2c \n
/// @param[in]  i2c library handle for Selected I2C channel ("I2C1" to "I2C4")
void eepromDemoOptionSelected(HANDLE i2c)
{
	DWORD returnValue = 0;
	BYTE eepromAddress = 0x50;                                               ///< Device address 
	BYTE eepromData[8] = {0x00, 0x00, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16};   ///< First two bytes denote the EEPROM address where data will be written, next 6 bytes are the data to be written
	BYTE eepromDataRead[8] = {0x0, 0x0, 0, 0, 0, 0, 0, 0};					 ///< Buffer for data 
	if(i2c == NULL)															 ///< Check for valid i2c handle									
	{
		printf("I2CDemo Error: Invalid i2c handle\n");
	}
	if(!I2c_SetConfigInt(i2c, L"SlaveAddr", eepromAddress, StoreVolatile))	 ///< Set Slave address for EEPROM
	{
		printf("I2CDemo Error: I2c_SetConfigInt, please verfy slave address\n" );
	}
	returnValue = I2c_Write(i2c, eepromData, 8);							 ///< Writes the 6 bytes of data to the device
	Sleep(10); 
	if(returnValue == I2C_RW_FAILURE)										 
	{
		printf("I2CDemo Error: I2c_Write\n");
	}
	else
	{
		printf("Data being written is %x %x %x %x %x %x\n", eepromData[2], eepromData[3], eepromData[4], eepromData[5], eepromData[6], eepromData[7]);

		returnValue = I2c_Write(i2c, eepromData, 2);							///< Writes the first two bytes of eepromData i.e. the address to the EEPROM
		Sleep(10); 
	}

	if(I2c_Read(i2c, (DWORD*) eepromDataRead, 6)!= I2C_RW_FAILURE)			///< To read the 6 bytes of data written at the given address (0x00)
	{
		printf("Data read is %x %x %x %x %x %x\n", eepromDataRead[0], eepromDataRead[1], eepromDataRead[2], eepromDataRead[3], eepromDataRead[4], eepromDataRead[5]);
	}
}

int wmain()
{ 
	HANDLE i2c = NULL;															///< i2C handle
	DWORD returnValue = 0;	
	tVersion ver;																///< Library version
	CHAR choice = 0;
	I2c_GetVersion(&ver);														///< Get I2C library vaersion
	printf("I2CDemo (I2CLib Version %d.%d.%d)\n", ver.Major, ver.Minor, ver.Build);
	i2c = I2c_Init(L"I2C1");													///< Init I2C handle 
	if (i2c == NULL)
	{
		printf("Error in I2c_Init()\r\n");
		return -1;
	}
	if(!I2c_Open(i2c))															///< Open I2C library 
	{
		printf("I2CDemo Error Open I2C\n");
	}
	if(!I2c_SetConfigInt(i2c, L"BitRateHz", 400000, StoreVolatile))				///< Set I2C speed to 400KHz
	{
		printf("I2CDemo Error Set config Bit Rate\n");
	}
	if(!I2c_SetConfigInt(i2c, L"SlaveAddrSize", 7, StoreVolatile))				///< Set 7 bit Slave address
	{
		printf("I2CDemo Error: I2c_SetConfigInt Slave addr size\n");
	}
	if(!I2c_SetConfigInt(i2c, L"RegisterAddrSize", 0, StoreVolatile))			///< No Register address to send
	{
		printf("I2CDemo Error: I2c_SetConfigInt RegisterAddr Size\n");
	}
	while(choice != EXIT_SELECTION)
	{
		fflush(stdin);
		printf("\nEnter option\n");
		printf("\na.) RTC \nb.) EEPROM(External EEPROM)\nc.) EXIT\n");
		scanf_s("%c",&choice);
		switch(choice)
		{
		case RTC_SELECTION:
			rtcDemoOptionSelected(i2c);											///< functon for RTC Demo 
			break;
		case EEPROM_SELECTION:
			eepromDemoOptionSelected(i2c);										///< function for EEPROM Demo
			break;
		case EXIT_SELECTION:
			printf("\nPress Enter to close the program\n");
			getchar();
			break;
		default:
			printf("Invalid option\n");
			break;
		}
	}
	I2c_Close(i2c);																///< Close and deinit I2C port
	I2c_Deinit(i2c);
	return TRUE;
}

